// INDEX.JS
// Cole Cassidy - 2022
// Express API router.

// external libraries
var cors = require('cors');
var bodyParser = require('body-parser');

// express unique import
var express = require('express'),
    app = express(),
    port = 3000;
app.use(cors());

// internal libraries
var iota = require('./iota.js');
const iotaDB = require('./dbConn.js');
const orgs = require('./organizations');
const users = require('./users');
const minutes = require('./minutes');
const auth = require('./authentication');
const validator = require('./validator');

// express configuration
app.use(express.json());
app.use(bodyParser.urlencoded({extended: false}));
app.use(bodyParser.json());

// test endpoint to validate server is up
app.get('/', async (req, res) => {
    try{
        await iota.testConn();
        res.send("Hello World");
        
    } catch(e) {
        console.log(e)
    }
    
});

// test endpoint 2 for routing
app.get('/auth', (req, res) => {
    res.send("Future Authentication Endpoint")
    console.log("Auth Request Received");
});

// User API
// Deprecated and replaced by the contents of users.js
// Kept here to demonstrate simple express behaviors. 
app.get('/user/:uid', async (req, res) => {
    try{
        console.log("User Request: "+req.params.uid+" From: "+req.ip);
        const userInfo = await users.findUser(req.params.uid);
        res.json(userInfo);
    } catch(e) {
        console.log(e);
    }
});

// Express routing table.
// Triggers route processing in imported files.
app.use('/org', orgs.router);
app.use('/usr', users.router);
app.use('/min', minutes.router);
app.use('/auth', auth.router);

// Waits to initialize the API server until DB connection is achieved. 
console.log("Waiting for DB Connection");
iotaDB.connect(() => {
    app.listen(port);
    console.log('API START');
});
