// VALIDATOR.JS
// Cole Cassidy - 2022
// Validates all tokens

// Imports
const express = require('express');

const jwt = require('jsonwebtoken');
const { ObjectId } = require('mongodb');

const iota = require('./iota');

// ========================================
//          Token Validation Functions
// ========================================

// Checks token validtity.
const checkToken = (req, res, next) => {
    
    // Extracts token from body or headers.
    const token = req.body.token || req.headers["x-access-token"];

    // If there is no token responds with error and ends request handling.
    if(token == null) return res.status(403).json(iota.ERR.RequestInvalid);

    // Validates token and appends user data to request for use in other functions in the callback chain.
    try {
        req.user = jwt.verify(token, iota.AUTHCONFIG.TokenKey);
    } catch(e){
        console.log(e);

        // If the extract fails the token is invalid. Responds with error and ends request handling.
        return res.status(401).json(iota.ERR.RequestInvalid);
    }
    
    // Executes next callback function.
    return next();
}

// Checks authentication for user.
// Does not validate tokens.
// If this function is called without first validating a token signature, hacked tokens can then execute privledged actions.
// ALWAYS VALIDATE FIRST.
const authUser = (req, res, next) => {

    // Extracts user and destingation data appended to request by checkToken();
    const user = req.user;
    const dest = req.body;

    // Log authentication attempt
    console.log(req.user);
    console.log(req.body);
    console.log(req.user.orgs.some(org => org.oid === req.body.oid));

    // If user has admin auth allow any action
    if(req.user.rank >= 3) return next();

    // If user is a member of affected group
    if( req.user.orgs.some(org => org.oid === req.body.oid)){
        return next();
    }

    // Same as previous statement, accounts for different methods of OID representation
    if( req.user.orgs.some(org => org.oid === new ObjectId(req.body.oid))){
        return next();
    }

    // Same as previous statement, used for GET requests and requests with no body.
    if(req.user.orgs.some(org => org.oid === req.params.oid)){
        console.log("hello ");
        return next();
    }

    // If user is affecting self
    if(req.user._id === ObjectId(req.body.uid)){
        return next();
    }

    // Otherwise terminate response with an unauthorized error.
    return res.status(401).json(iota.ERR.Unauthorized);

}

// EXPORTS
module.exports = {
    checkToken,
    authUser
}